// ===== Modellierung ========== Wolfgang Erben

package pruefung;
import static org.junit.Assert.*;
import org.junit.Test;
import pruefung.version2.Polynom;
import java.util.*;

public class PolynomTest {
    @Test
    public void testeKonstantZufaellig() {
        final int iterationen = 1000;
        Random r = new Random();
        final int a = 87;
        for (int i = 0; i < iterationen; i++) {
            double x = r.nextDouble();
            Polynom p1 = new Polynom();
            assertEquals(-1, p1.grad());
            assertEquals(0., p1.wertBei(x), 0.);
            Polynom p2 = new Polynom(a);
            assertEquals(0, p2.grad());
            assertEquals(a, p2.wertBei(x), 0.);
        }
    }
    @Test
    public void testeMonomeElementar() {
        int[] koeffGrad0 = { 1 };
        double[] xWerte = { 0., 1., 2., 3. };
        for (int i = 0; i < xWerte.length; i++) {
            double x = xWerte[i];
            double y = 1.;
            for (int grad = 1; grad < 7; grad++) {
                int[] koeff = Arrays.copyOf(koeffGrad0, grad + 1);
                Polynom p = new Polynom(koeff);
                assertEquals(grad, p.grad());
                y *= x;
                assertEquals(y, p.wertBei(x), 0.);
            }
        }
    }
    @Test
    public void testeUnveraenderbarkeitKonstruktor() {
        int a = 1;
        int[] koeff = { a };
        Polynom p = new Polynom(koeff);
        assertEquals(0, p.grad());
        assertEquals(a, p.wertBei(0.), 0.);
        koeff[0] = 7;
        assertEquals(a, p.wertBei(0.), 0.);
    }
    @Test
    public void testeEqualsEinfach() {
        for (int a = -99; a < 100; a++)
            for (int b = -99; b < 100; b++) {
                Polynom p = new Polynom(a, b);
                Polynom q = new Polynom(a, b);
                assertEquals(p, q);
                assertEquals(p.grad(), q.grad());
                assertEquals(p.hashCode(), q.hashCode());
                assertEquals(p.toString(), q.toString());
            }
    }
    @Test
    public void testeSumme() {
        Polynom p1 = new Polynom(1, 2, 1);
        Polynom p2 = new Polynom(-1, 2, -1);
        Polynom summe = p1.plus(p2);
        assertEquals(1, summe.grad());
    }
    @Test
    public void testeStammfunktion() {
        Polynom p = new Polynom(3, 0, 7); // 3x^2 + 7
        Funktion ist = p.stammFunktion(); // Stammfunktion mit Nullstelle bei 0
        Funktion soll = new Polynom(1, 0, 7, 0); // x^3 + 7x
        final double tol = 0.02;
        double[] xWerte = { -1., 0., 1., 2. };
        for (double x : xWerte)
            assertEquals(soll.wertBei(x), ist.wertBei(x), tol);
    }
}
