// ===== Wolfgang Erben ========== Modellierung =====
//
package pruefung;

/**
 * Die Dekorierer-Klasse <b>StammFunktion</b> beschreibt Stamm-Funktionen
 * reellwertiger Funktionen (eines reellen Parameters).
 * 
 * @author Wolfgang Erben
 * @version 1.0 6.4.2011
 */
public final class StammFunktion implements Funktion {
    private final double mA;
    private final Funktion mFunktion;
    private static final int ANZAHL_INTERVALLE = 1000;
    private static final int E = 100;
    
    /**
     * erzeugt eine Stammfunktion der angegebenen Funktion,
     * welche an der angegebenen Stelle den Wert 0 hat.
     */
    public StammFunktion(Funktion f, double a) {
        mFunktion = f;
        mA = a;
    }

    /**
     * liefert immer <code>true</code>.
     */
    @Override
    public boolean istDefiniertBei(double x) {
        return true;
    }

    /**
     * liefert eine Stammfunktion dieser Stammfunktion.
     */
    @Override
    public Funktion stammFunktion() {
        return new StammFunktion(this, mA);
    }

    /**
     * liefert den Wert der Stammfunktion an der angegebenen Stelle.
     */
    @Override
    public double wertBei(double x) {
        return Funktionen.riemannSummeVon(mFunktion, mA, x, ANZAHL_INTERVALLE);
    }

}
