// ===== Modellierung ========== Wolfgang Erben

package pruefung.version1;
import pruefung.*;

public final class Polynom extends UeberallDefinierteFunktion {
    private final int[] mKoeff;
    public Polynom(int... koeff) {
        if (koeff.length == 0) throw new IllegalArgumentException();
        mKoeff = koeff;
    }
    public Polynom() {
        mKoeff = new int[0];
    }
    public int grad() {
        return mKoeff.length - 1; 
    }
    @Override
    public String toString() {
        return "Polynom" + mKoeff;
    }
    @Override
    public double wertBei(double x) {
        double erg = 0.;
        for (int a : mKoeff) erg = erg * x + a;
        return erg;
    }
    @Override
    public boolean equals(Object o) {
        if (o == this) return true;
        if (o == null) return false;
        if (!(o instanceof Polynom)) return false;
        Polynom p = (Polynom) o;
        if (p.grad() != grad()) return false;
        for (int i = 0; i < mKoeff.length; i++)
            if (p.mKoeff[i] != mKoeff[i]) return false;
        return true;
    }
    public Polynom plus(Polynom p) {
        int grad = Math.max(mKoeff.length, p.mKoeff.length);
        int[] koeff = new int[grad];
        for (int i = 0; i < grad; i++)
            koeff[i] = mKoeff[i] + p.mKoeff[i];
        return new Polynom(koeff);
    }
    public Polynom mal(Polynom p) {
        int grad = grad() + p.grad();
        int[] koeff = new int[grad];
        for (int i = 0; i < grad; i++) {
            koeff[i] = 0;
            for (int j = 0; j < i; j++)
                koeff[i] += mKoeff[j] * p.mKoeff[i - j];
        }
        return new Polynom(koeff);
    }
    public Polynom hoch(int n) {
        if (n < 0) throw new IllegalArgumentException();
        Polynom erg = new Polynom(1);
        for (int i = 0; i < n; i++)
            erg = erg.mal(this);
        return erg;
    }
}
